﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Diagnostics;
using System.IO;
using System.Windows;
using ThinkGeo.Core;

namespace Demo
{
    /// <summary>
    /// Interaction logic for MainWindow.xaml
    /// </summary>
    public partial class MainWindow
    {
        private string wellDepthPointDataFilePath = @"data\PTALResult1.csv";
        private Dictionary<PointShape, double> _wellDepthPointData;
        private GridCell[,] _gridCellMatrix;
        private readonly string[] _colorStrings = {
            "#333333",
            "#000080",
            "#0000ff",
            "#00ffff",
            "#339a65",
            "#ffff00",
            "#ff9a00",
            "#ff0000",
            "#800000"
        };

        readonly List<double> _isoLineLevels = new List<double>
        {
            0.01     ,
            2.5      ,
            5.00     ,
            10.0     ,
            15.0     ,
            20.0     ,
            25.0     ,
            40.0     ,
            100000.0 ,
        };
        private readonly RectangleShape _extent =
            new PolygonShape("POLYGON((397746 302890,397746 275390,418046 275390,418046 302890,397746 302890))")
                .GetBoundingBox();

        public MainWindow()
        {
            InitializeComponent();
        }

        private void MapView_OnLoaded(object sender, RoutedEventArgs e)
        {
            mapView.MapUnit = GeographyUnit.Meter;
            mapView.CurrentExtent = _extent;

            //Load the well depth points and depth data from a text file into the dictionary
            //We cache this at the class level to prevent form loading it multiple times
            _wellDepthPointData = GetWellDepthPointDataFromCSV(wellDepthPointDataFilePath);


            //Add the grid layer, the grid cells, and the well points to the map
            LayerOverlay isoLineOverlay = new LayerOverlay();

            mapView.Overlays.Add("isoLineOverlay", isoLineOverlay);

            isoLineOverlay.Layers.Add("IsoLineLayer", GetGridIsoLineLayer(IsoLineType.ClosedLinesAsPolygons));
            isoLineOverlay.DrawingQuality = DrawingQuality.HighQuality;

            mapView.Refresh();
        }

        private static Dictionary<PointShape, double> GetWellDepthPointDataFromCSV(string csvFilePath)
        {
            StreamReader streamReader = null;
            var wellDataPoints = new Dictionary<PointShape, double>();

            try
            {
                streamReader = new StreamReader(csvFilePath);
                var headline = streamReader.ReadLine();
                while (!streamReader.EndOfStream)
                {
                    var line = streamReader.ReadLine();

                    var parts = line.Split(',');
                    wellDataPoints.Add(new PointShape(double.Parse(parts[1]), double.Parse(parts[2])), double.Parse(parts[3]));
                }
            }
            finally
            {
                streamReader?.Close();
            }
            return wellDataPoints;
        }

        private void CreateGridCellMatrix()
        {
            //Get the current extent since we use that to gerenate the grid.  Of course this is just for
            //the demo and in the real world you can use any extent you like
            RectangleShape currentDrawingExtent = _extent;

            //Calculate the cell size based on how many rows and columns you specified
            double cellSize = 100;
            //Greate the grid definition based on the extent, cell size etc.
            GridDefinition gridDefinition = new GridDefinition(currentDrawingExtent, cellSize, -9999, _wellDepthPointData);

            //Generate the grid based on Inverse Distance Weighted interpolation model.  You can define your own model if needed.
            _gridCellMatrix = GridFeatureSource.GenerateGridMatrix(gridDefinition, new InverseDistanceWeightedGridInterpolationModel(5, cellSize * 4));
        }


        private InMemoryGridIsoLineLayer GetGridIsoLineLayer(IsoLineType type)
        {
            //Create a grid cell matrix based on the textboxes and the current extent
            CreateGridCellMatrix();

            //Load the grid we just created into the GridIsoLineLayer using the number of breaks defines
            //on the screen

            var isoLineLayer = new SqlTypesIsoLineLayer(_gridCellMatrix, _isoLineLevels)
            {
                IsoLineType = type
            };

            //Setup a class break style based on the isoline levels and the colors
            var classBreakStyle = new ClassBreakStyle(isoLineLayer.DataValueColumnName);

            var firstStyles = new Collection<ThinkGeo.Core.Style>
            {
                new LineStyle(new GeoPen(GeoColor.FromHtml(_colorStrings[0]), 3)),
                new AreaStyle(new GeoPen(GeoColors.LightBlue, 1),
                    new GeoSolidBrush(GeoColor.FromHtml(_colorStrings[0])))
            };
            classBreakStyle.ClassBreaks.Add(new ClassBreak(double.MinValue, firstStyles));
            for (var i = 0; i < _isoLineLevels.Count - 1; i++)
            {
                var styles = new Collection<ThinkGeo.Core.Style>
                {
                    new LineStyle(new GeoPen(GeoColor.FromHtml(_colorStrings[i + 1]), 3)),
                    new AreaStyle(new GeoPen(GeoColors.LightBlue, 1),
                        new GeoSolidBrush(GeoColor.FromHtml(_colorStrings[i + 1])))
                };
                classBreakStyle.ClassBreaks.Add(new ClassBreak(_isoLineLevels[i], styles));
            }
            isoLineLayer.CustomStyles.Add(classBreakStyle);

            return isoLineLayer;
        }
    }
}
